;****************************************************************
;								*
;  QMUL  -  Quick 16 x 16 Multiply				*
;								*
;===============================================================*
;								*
;         By:           Jon Tara				*
;                      	MicroControl Software			*
;			1300 E. Lafayette #2808			*
;			Detroit, Michigan  48207		*
;			(313) 393-2916				*
;								*
;        This software is placed in the public domain.		*
;                                                               *
; Bugs in code fixed by Jeff Frohwein, 25-Apr-97                *
;								*
;===============================================================*
;								*
;  On entry:  DE = Multiplicand					*
;	      HL = Multiplier					*
;								*
;  On exit:   BCDE = Product					*
;								*
;  AF destroyed							*
;								*
;  Revision 1.00  01/21/82  JT  Initial Release			*
;								*
;****************************************************************

.BANK 0
.ORG $150

.SECTION "QUICK_MULTIPLY"

QMUL:   push    hl              ; Save multiplier.
        ld      c, h            ; Save MSBs of multiplier.
        ld      a, l            ; LSBs to A for an 8 x 16 multiply.

        ld      b, 0            ; Handy 0 to B for carry propagation.
        ld      h, b            ; Init LSBs of product.
        ld      l, b

        add     a               ; Test multiplier bit.
        jr      nc, _QMUL_M1
        add     hl, de          ; Add multiplicand to product.
        adc     b               ; (Product in AHL)
_QMUL_M1:
        add     hl, hl          ; Shift product left.
        adc     a               ; Test multiplier bit.
        jr      nc, $4
        add     hl, de          ; Add multiplicand to product.
        adc     b               ; (Product in AHL)
        add     hl, hl          ; Shift product left.
        adc     a               ; Test multiplier bit.
        jr      nc, $4
        add     hl, de          ; Add multiplicand to product.
        adc     b               ; (Product in AHL)
        add     hl, hl          ; Shift product left.
        adc     a               ; Test multiplier bit.
        jr      nc, $4
        add     hl, de          ; Add multiplicand to product.
        adc     b               ; (Product in AHL)
        add     hl, hl          ; Shift product left.
        adc     a               ; Test multiplier bit.
        jr      nc, $4
        add     hl, de          ; Add multiplicand to product.
        adc     b               ; (Product in AHL)
        add     hl, hl          ; Shift product left.
        adc     a               ; Test multiplier bit.
        jr      nc, $4
        add     hl, de          ; Add multiplicand to product.
        adc     b               ; (Product in AHL)
        add     hl, hl          ; Shift product left.
        adc     a               ; Test multiplier bit.
        jr      nc, $4
        add     hl, de          ; Add multiplicand to product.
        adc     b               ; (Product in AHL)
        add     hl, hl          ; Shift product left.
        adc     a               ; Test multiplier bit.
        jr      nc, $4
        add     hl, de          ; Add multiplicand to product.
        adc     b               ; (Product in AHL)

        push    hl              ; Save LSBs in stack.
        ld      h, b            ; Zero second product.
        ld      l, b            ; .
        ld      b, a            ; Save MSBs of first product in B
        ld      a, c            ; Get MSBs of multiplier.
        ld      c, h            ; Handy 0 in C this time.
 
        add     a               ; Test multiplier bit.
        jr      nc, $4
        add     hl, de          ; Add multiplicand to product.
        adc     c               ; (Product in AHL)
        add     hl, hl          ; Shift product left.
        adc     a               ; Test multiplier bit.
        jr      nc, $4
        add     hl, de          ; Add multiplicand to product.
        adc     c               ; (Product in AHL)
        add     hl, hl          ; Shift product left.
        adc     a               ; Test multiplier bit.
        jr      nc, $4
        add     hl, de          ; Add multiplicand to product.
        adc     c               ; (Product in AHL)
        add     hl, hl          ; Shift product left.
        adc     a               ; Test multiplier bit.
        jr      nc, $4
        add     hl, de          ; Add multiplicand to product.
        adc     c               ; (Product in AHL)
        add     hl, hl          ; Shift product left.
        adc     a               ; Test multiplier bit.
        jr      nc, $4
        add     hl, de          ; Add multiplicand to product.
        adc     c               ; (Product in AHL)
        add     hl, hl          ; Shift product left.
        adc     a               ; Test multiplier bit.
        jr      nc, $4
        add     hl, de          ; Add multiplicand to product.
        adc     c               ; (Product in AHL)
        add     hl, hl          ; Shift product left.
        adc     a               ; Test multiplier bit.
        jr      nc, $4
        add     hl, de          ; Add multiplicand to product.
        adc     c               ; (Product in AHL)
        add     hl, hl          ; Shift product left.
        adc     a               ; Test multiplier bit.
        jr      nc, $4
        add     hl, de          ; Add multiplicand to product.
        adc     c               ; (Product in AHL)

        pop     de              ; Fetch LSBs of 1st product.
        ld      c, a            ; Add partial products.
        ld      a, d            ; .
        add     l               ; .
        ld      d, a            ; .
        ld      a, b            ; .
        adc     h               ; .
        ld      h, a            ; .
        ld      a, c            ; .
        adc     0               ; .
        ld      b, a            ; .
        ld      c, h            ; .
        pop     hl              ; Restore multiplier.
        ret

.ENDS
